#ifndef _CMDLINE_CODE
#define _CMDLINE_CODE

#include <stdlib.h>   // atoi()
#include <stdio.h>    // printf(), getc()
#include <string.h>   // strupr(), strcmp()

#include "config.h"   // Config structure
#include "cmdline.h"

extern char* pConfig; // default configuration file name (i.e. beautcpp.cfg)

// This function displays brief command line help to the user.
// Parameters:
// char* argv[]     : Pointer to command line parameter pointer array
//
void PrintProgramHelp (char* argv[])
{
    int userKey;

    printf ("\nC++ Beautifier     V1.5\n");
    printf ("-----------------------\n\n");
    printf ("Program Was Written By Steven De Toni, Copyright October 1994\n");
    printf ("This Program Is The Result Of A Small C++ Project;\n");
    printf ("The Executable, And Code Are Public Domain.\n\n");

    printf ("Usage : %s [ parameters ]\n\n", argv[0]);
    printf ("            Available Command Line Parameters\n");
    printf ("            ---------------------------------\n\n");
    printf ("[-f <num>] [-t]   [-i <num>] [-cc <num>] [-nc <num>]    [-na]\n");
    printf ("[-nq]      [-lg]  [-bcl]     [-bnl]      [-fi <string>] [-fo <string>]\n");
    printf ("[-fc]      [-fnc] [-no]      [-qb <num>] [-h] [-?]\n");
    printf ("\n[] = Optional <> = Parameter Requirement\n");
    printf ("\n More Detail Y/N ? ");

    userKey = getc(stdin);
    userKey = (userKey & 223); // change key to upper case!
    if (userKey == 'Y')
    {
        printf ("\n-f   <num>    : -f 20       : Function Line Spacing.\n");
        printf ("-t            : -t          : Use Tabs In Indenting.\n");
        printf ("-i   <num>    : -i 4        : Indent Space Length.\n");
        printf ("-cc  <num>    : -cc 50      : Comments With Code.\n");
        printf ("-nc  <num>    : -nc 0       : Comments With No Code.\n");
        printf ("-na           : -na         : Remove Non-Ascii chars.\n");
        printf ("-nq           : -nq         : Convert Non-Ascii Chars In Quotes To Octal.\n");
        printf ("-lg           : -lg         : Leave Graphic Chars.\n");
        printf ("-bcl          : -bcl        : Open Braces On Code Line.\n");
        printf ("-bnl          : -bnl        : Open Braces On New Line (default).\n");
        printf ("-fi  <string> : -fi in.cpp  : Input File Name.\n");
        printf ("-fo  <string> : -fo out.cpp : Output File Name.\n");
        printf ("-fc           : -fc         : Use Configuration File In Current Directory.\n");
        printf ("-fnc <string> : -fnc c.cfg  : Load Custom Configuration File.\n");
        printf ("-no           : -no         : Suppress Program Output (Unless Errors Occur).\n");
        printf ("-qb  <num>    : -qb 20      : Define Internal Queue Buffer.\n");
        printf ("-h    or   -? : -h   or  -? : This Help.\n");
        printf ("\n* Support For I/O Redirection Is Provided *\n");
    }
}

// integer assignment
int intAssign (int& cmdCount, int argc, char* argv[] )
{
    if (cmdCount+1 <= (argc-1))
    {
        cmdCount++;
        return atoi (argv[cmdCount]);
    }
    else
    {
        printf ("Expected Another Integer Parameter For Command Directive %s\n", argv[cmdCount]);
        PrintProgramHelp (argv);
        cmdCount = -1;
        return cmdCount;
    }
}

char* strAssign (int& cmdCount, int argc, char* argv[])
{
    if (cmdCount+1 <= (argc-1))
    {
        cmdCount++;
        return argv[cmdCount];
    }
    else
    {
        printf ("Expected Another String Parameter For Command Directive %s\n", argv[cmdCount]);
        PrintProgramHelp (argv);
        cmdCount = -1;
        return NULL;
    }
}

// Function processes command line parameters
int ProcessCommandLine (int argc, char* argv[], Config& settings,
                        char*& pInFile, char*& pOutFile)
{

    // set defaults
    settings.numOfLineFunc    = 2    ;// number of lines between functions
    settings.useTabs          = False;// use tabs to indents rather than spaces
    settings.tabSpaceSize     = 4    ;// number of spaces a tab takes up {4}
    settings.posOfCommentsWC  = 50   ;// position of comments on line with code
    settings.posOfCommentsNC  = 0    ;// position of comments on line
    settings.quoteChars       = False;// use tabs to indents rather than spaces
    settings.deleteHighChars  = 0    ;// 0  = no check         , 1 = delete high chars,
                                      // 3  = delete high chars, but not graphics
    settings.braceLoc         = True ;// Start open braces on new line
    settings.output           = True ;// Set this true for normal program output
    settings.queueBuffer      = 10   ;// Set the number if lines to store in memory at a time !

    int   cmdCount = 0;
    char* cmdRead;

    char* pBkConfig = pConfig;
    pConfig         = NULL;

    while (cmdCount < argc-1)
    {
        // command line error
        if (cmdCount < 0)
           return cmdCount;

        // next command to process!
        cmdCount++;
        cmdRead = argv[cmdCount];

        // upcase the command parameter
        strupr (cmdRead);

        if (cmdRead[0] == '-') // this is a command directive
        {
            cmdRead++;

            // Test for "No Output Wanted"
            if (strcmp ("NO", cmdRead) == 0)
            {
                settings.output = False;
                continue;
            }

            // Test for "Comments With No Code Position"
			if (strcmp ("NC", cmdRead) == 0)
            {
                settings.posOfCommentsNC = intAssign (cmdCount, argc, argv);
                continue;
            }

            // Test for "Remove Non-Ascii Chars"
            if (strcmp ("NA", cmdRead) == 0)
            {
                settings.deleteHighChars = 1;
                continue;
            }

            // Test for "Convert Non-Ascii Chars In Quotes To Octal"
            if (strcmp ("NQ", cmdRead) == 0)
            {
                settings.quoteChars = True;
                continue;
            }

            // Test for "Leave Graphic Chars"
            if (strcmp ("LG", cmdRead) == 0)
            {
                settings.deleteHighChars = 3;
                continue;
            }

            // Test for "Comments With Code Position"
            if (strcmp ("CC", cmdRead) == 0)
            {
                settings.posOfCommentsWC = intAssign (cmdCount, argc, argv);
                continue;
            }

             // Test for  "Read Configuration File"
            if (strcmp ("FC", cmdRead) == 0)
            {
                pConfig = pBkConfig;
                continue;
            }

            // Test for  "File Name To Read New Configuration File"
            if (strcmp ("FNC", cmdRead) == 0)
            {
                pConfig = strAssign (cmdCount, argc, argv);
                continue;
            }

            // Test for "Input File Name"
            if (strcmp ("FI", cmdRead) == 0)
            {
                pInFile = strAssign (cmdCount, argc, argv);
                continue;
            }

            // Test for "Output File Name"
            if (strcmp ("FO", cmdRead) == 0)
            {
                pOutFile = strAssign (cmdCount, argc, argv);
                continue;
            }

            // Test for "Function Spacing"
            if (strcmp ("F", cmdRead) == 0)
            {
                settings.numOfLineFunc = intAssign (cmdCount, argc, argv);
                continue;
            }

            // Test for "Use Tabs In Indenting"
            if (strcmp ("T", cmdRead) == 0)
            {
                settings.useTabs = True;
                continue;
            }

            // Test for "Indent Spacing Length"
            if (strcmp ("I", cmdRead) == 0)
            {
                settings.tabSpaceSize = intAssign (cmdCount, argc, argv);
                continue;
            }

            // Test for "Open Brace On Code Line"
            if (strcmp ("BCL", cmdRead) == 0)
            {
                settings.braceLoc = False;
                continue;
            }

            // Test for "Open Brace On New Line"
            if (strcmp ("BNL", cmdRead) == 0)
            {
                settings.braceLoc = True;
                continue;
            }

            // Test for "Queue Buffer"
            if (strcmp ("QB", cmdRead) == 0)
            {
                settings.queueBuffer = intAssign (cmdCount, argc, argv);

                if (settings.queueBuffer < 2)
                   settings.queueBuffer = 2;

                continue;
            }

            // ### display help ###
            if( (strcmp ("?", cmdRead) == 0) ||
                (strcmp ("H", cmdRead) == 0) )
            {
                printf ("*** Displaying Brief Help ***\n");
                PrintProgramHelp (argv);
                return -1;
            }

            printf ("Unkown Command Directive %s \n", cmdRead);
            PrintProgramHelp (argv);
            return -1;
        }
        else
        {
            printf ("Command Line Error : Expected Command Directive For %s.\n", argv[cmdCount]);
            PrintProgramHelp (argv);
            return -1;
        }
    }

    return 0;
}

#endif _CMDLINE_CODE